import logging


class LoggingMixin:
    """ Convenience super-class to have a logger configured with the class name
    """

    def __init__(self, context=None):
        self._set_context(context)

    @property
    def log(self) -> logging.Logger:
        """ A logger object.
        Returns:
            Logger
        """
        self._log = logging.getLogger(
            self.__class__.__module__ + "." + self.__class__.__name__
        )
        # self.log.setup_logging(log_level=10)
        return self._log

    def _set_context(self, context):
        if context is not None:
            set_context(self.log, context)


def set_context(logger, value):
    """ Walks the tree of loggers and tries to set the context for each handler.
    
    Args:
        logger: logger to traverse
        value: value to set
    """
    _logger = logger
    while _logger:
        for handler in _logger.handlers:
            try:
                handler.set_context(value)
            except AttributeError:
                # Not all handlers need to have context passed in so we ignore
                # the error when handlers do not have set_context defined.
                pass
        if _logger.propagate is True:
            _logger = _logger.parent
        else:
            _logger = None




from slack_sdk.web import WebClient
from logging import StreamHandler
import logging
import os


class SlackHandler(StreamHandler):
    def __init__(self, channel: str, slack_bot_token: str = None):
        StreamHandler.__init__(self)
        # Initialize a Web API client
        if slack_bot_token:
            self.slack_web_client = WebClient(token=slack_bot_token)
        else:
            self.slack_web_client = WebClient(token=os.environ["SLACK_BOT_TOKEN"])
        self.channel = channel

    def _send_log(self, message: str, channel: str):
        print("Sending log")
        # Create a new onboarding tutorial.
        onboarding_tutorial = LogMessage(channel, message)

        # Get the onboarding message payload
        slack_payload = onboarding_tutorial.get_message_payload()

        # Post the onboarding message in Slack
        response = self.slack_web_client.chat_postMessage(**slack_payload)

    def emit(self, message: str):
        assert isinstance(message, logging.LogRecord)

        self.format(message)

        # List of LogRecord attributes expected when reading the
        # documentation of the logging module:

        expected_attributes = (
            "args,asctime,created,exc_info,filename,funcName,levelname,"
            "levelno,lineno,module,msecs,message,msg,name,pathname,"
            "process,processName,relativeCreated,stack_info,thread,threadName"
        )

        for ea in expected_attributes.split(","):
            if not hasattr(message, ea):
                print("UNEXPECTED: LogRecord does not have the '{}' field!".format(ea))

        self._send_log(message, self.channel)


class LogMessage:
    """Constructs the onboarding message and stores the state of which tasks were completed."""

    ERROR_TYPES_EMOJI = {
        "CRITICAL": ":interrobang:",
        "ERROR": ":red_circle:",
        "WARNING": ":large_orange_diamond:",
        "INFO": ":large_blue_circle:",
        "DEBUG": ":white_circle:",
    }

    DIVIDER_BLOCK = {"type": "divider"}

    def __init__(self, channel, message):
        self.channel = channel
        self.message = message
        self.username = "pythonboardingbot"
        self.icon_emoji = self.ERROR_TYPES_EMOJI[message.levelname]
        self.timestamp = ""
        self.reaction_task_completed = False
        self.pin_task_completed = False

    def get_message_payload(self):
        return {
            "ts": self.timestamp,
            "channel": self.channel,
            "username": self.username,
            "icon_emoji": self.icon_emoji,
            "blocks": [*self._create_log_block(), self.DIVIDER_BLOCK,],
        }

    def _create_log_block(self):
        emoji = f"{self.ERROR_TYPES_EMOJI[self.message.levelname]}"
        text = f"```{self.message.getMessage()}```"
        information = f"{self.message.asctime}-{self.message.name}[{self.message.process}]-{self.message.module}-{self.message.levelname}"
        return self._get_log_block(emoji, text, information)

    @staticmethod
    def _get_log_block(emoji, text, information):
        return [
            {"type": "section", "text": {"type": "mrkdwn", "text": emoji}},
            {"type": "context", "elements": [{"type": "mrkdwn", "text": information}],},
            {"type": "section", "text": {"type": "mrkdwn", "text": text}},
        ]



#!/usr/bin/env python

import os
import re
import sys

from codecs import open

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

repo_path = os.path.abspath(os.path.dirname(__file__))

packages = find_packages(exclude=("examples",))

with open("requirements.txt") as f:
    install_requires = [line for line in f if "==" in line]

about = {}
with open(os.path.join(repo_path, "gytrash", "__about__.py"), "r", "utf-8") as f:
    exec(f.read(), about)
with open("README.md", "r", "utf-8") as f:
    readme = f.read()

setup(
    name=about["__title__"],
    version="0.0.9",
    description=about["__description__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    packages=packages,
    package_dir={"gytrash": "gytrash"},
    package_data={"": ["*.cwl", "*.yaml"]},
    include_package_data=True,
    python_requires=">=3.7",
    install_requires=install_requires,
    setup_requires=install_requires,
    license=about["__license__"],
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    tests_require=[],
    project_urls={"Source": "https://github.com/trejas/gytrash"},
)
